unit HyperlinkLabel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  THyperlinkLabel = class(TLabel)
  private
    FHyperlinkPrefix: string;
    FClickedHyperlinkColor: TColor;
    FTargetURL: string;
    FUseClickedHyperlinkColor: boolean;
    FUseHoverColor: boolean;
    FHoverColor: TColor;

    OriginalFontColor: TColor;
    OriginalFontStyle: TFontStyles;
    FUseHoverFontStyle: boolean;
    FHoverFontStyle: TFontStyles; // In hover-behaviour implementation.

    procedure SetHyperlinkPrefix(const Value: string);
    procedure SetClickedHyperlinkColor(const Value: TColor);
    procedure SetTargetURL(const Value: string);
    procedure SetUseClickedHyperlinkColor(const Value: boolean);
    procedure SetHoverColor(const Value: TColor);
    procedure SetUseHoverColor(const Value: boolean);
    procedure SetHoverFontStyle(const Value: TFontStyles);
    procedure SetUseHoverFontStyle(const Value: boolean);
  protected
    procedure Click; override;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Navigate;
  published
    property TargetURL: string read FTargetURL write SetTargetURL;
    property HyperlinkPrefix: string read FHyperlinkPrefix write SetHyperlinkPrefix;
    property ClickedHyperlinkColor: TColor read FClickedHyperlinkColor write SetClickedHyperlinkColor;
    property UseClickedHyperlinkColor: boolean read FUseClickedHyperlinkColor write SetUseClickedHyperlinkColor;
    property HoverColor: TColor read FHoverColor write SetHoverColor;
    property UseHoverColor: boolean read FUseHoverColor write SetUseHoverColor;
    property HoverFontStyle: TFontStyles read FHoverFontStyle write SetHoverFontStyle;
    property UseHoverFontStyle: boolean read FUseHoverFontStyle write SetUseHoverFontStyle;
  end;

  // TODO: How to show the new window (SW_SHOW,SW_SHOWMAXIMIZED,... parameter of ShellExecute).
  // TODO: Show command: SW_SHOW, SW_MAXIMIZE,...

procedure Register;

implementation

uses
  ShellAPI;

{$R *.dcr}

procedure Register;
begin
  RegisterComponents('BUtil', [THyperlinkLabel]);
end;

{ THyperlinkLabel }

procedure THyperlinkLabel.Click;
begin
  inherited;
  Navigate
end;

procedure THyperlinkLabel.Navigate;
var
  Tmp: string;
begin

  // If user creates hyperlink label in run-time.
  if not Assigned(Parent) then
    raise Exception.Create('THyperlinkLabel.Navigate() Parent must be assigned.');

  if TargetURL = '' then
    Tmp := HyperlinkPrefix + Caption
  else
    Tmp := HyperlinkPrefix + TargetUrl;

  if ShellExecute(Parent.Handle,'open',PChar(Tmp),nil,nil,SW_SHOW) > 32 then
    if UseClickedHyperlinkColor then
    begin
      OriginalFontColor := ClickedHyperlinkColor; // Just in case.
      Font.Color := ClickedHyperlinkColor;
    end

end;

constructor THyperlinkLabel.Create(AOwner: TComponent);
begin
  inherited;
  Font.Color := clBlue;
  Font.Style := [fsUnderline];
  Cursor := crHandPoint;
  ClickedHyperlinkColor := clPurple;
  UseClickedHyperlinkColor := true;
  HoverColor := clRed;
  UseHoverColor := true;
  HoverFontStyle := [fsUnderline];
  UseHoverFontStyle := false
end;

procedure THyperlinkLabel.CMMouseEnter(var Message: TMessage);
begin
  OriginalFontColor := Font.Color;
  OriginalFontStyle := Font.Style;
  if UseHoverColor then
    Font.Color := HoverColor;
  if UseHoverFontStyle then
    Font.Style := HoverFontStyle;
end;

procedure THyperlinkLabel.CMMouseLeave(var Message: TMessage);
begin
  if UseHoverColor then
    Font.Color := OriginalFontColor;
  if UseHoverFontStyle then
    Font.Style := OriginalFontStyle
end;

procedure THyperlinkLabel.SetTargetURL(const Value: string);
begin
  FTargetURL := Value;
end;

procedure THyperlinkLabel.SetHyperlinkPrefix(const Value: string);
begin
  FHyperlinkPrefix := Value;
end;

procedure THyperlinkLabel.SetClickedHyperlinkColor(const Value: TColor);
begin
  FClickedHyperlinkColor := Value;
end;

procedure THyperlinkLabel.SetUseClickedHyperlinkColor(
  const Value: boolean);
begin
  FUseClickedHyperlinkColor := Value;
end;

procedure THyperlinkLabel.SetHoverColor(const Value: TColor);
begin
  FHoverColor := Value;
end;

procedure THyperlinkLabel.SetUseHoverColor(const Value: boolean);
begin
  FUseHoverColor := Value;
end;

procedure THyperlinkLabel.SetHoverFontStyle(const Value: TFontStyles);
begin
  FHoverFontStyle := Value;
end;

procedure THyperlinkLabel.SetUseHoverFontStyle(const Value: boolean);
begin
  FUseHoverFontStyle := Value;
end;



end.
