unit InterbaseAutostart;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TCustomAutostart = class(TComponent)
  protected
    function IsStarted: boolean; virtual; abstract;
    procedure Start; virtual; abstract;
  public
    constructor Create(AOwner: TComponent); override;
  end;

  TCustomInterbaseAutostart = class(TCustomAutostart)
  protected
    procedure IBStart(const Guardian: boolean);
    function IsStarted: boolean; override;
  end;

  TInterbaseAutostart = class(TCustomInterbaseAutostart)
  protected
    procedure Start; override;
  end;

  TInterbaseGuardianAutostart = class(TCustomInterbaseAutostart)
  protected
    procedure Start; override;
  end;

procedure Register;

implementation

{$R *.DCR}

uses
  registry,BUtil_Shared;

const

  // Maximal time for Interbase start in milliseconds. If this time is
  // exceeded, autostart fails. You can place INFINITE here for infinite
  // timeout interval.
  IBStartTimeout = 20000;

  // How much to wait after guardian start (in milliseconds).
  AfterGuardianTimeout = 500;


procedure Register;
begin
  RegisterComponents('BUtil', [TInterbaseAutostart,TInterbaseGuardianAutostart]);
end;

{ TInterbaseStart }

constructor TCustomAutostart.Create(AOwner: TComponent);
begin
  inherited;
  //if not (csDesigning in ComponentState) and not IsStarted then
  if not IsStarted then
    Start;
end;

{ TInterbaseAutostart }

function TCustomInterbaseAutostart.IsStarted: boolean;
var
  WindowHandle: THandle;
begin
  WindowHandle := FindWindow('IB_Server',nil);
  try
    result := WindowHandle <> 0;
  finally
    CloseHandle(WindowHandle)
  end
end;

(*

-hwnd -> proces id
DWORD GetWindowThreadProcessId(
  HWND hWnd,	// handle of window
  LPDWORD lpdwProcessId 	// address of variable for process identifier
);

-proces id -> proces handle
HANDLE OpenProcess(
  DWORD dwDesiredAccess,	// access flag
  BOOL bInheritHandle,	// handle inheritance flag
  DWORD dwProcessId 	// process identifier
);

*)

procedure TCustomInterbaseAutostart.IBStart(const Guardian: boolean);
var
  R: TRegistry;
  ServerDirectory,Exe: string;
  ProcessHandle: THandle;
begin

  R := TRegistry.Create;
  try
    R.RootKey := HKEY_LOCAL_MACHINE;
    {$IFDEF VER110}
    R.OpenKey('Software\InterBase Corp\InterBase\CurrentVersion',false);
    {$ELSE}
    R.OpenKeyReadOnly('Software\InterBase Corp\InterBase\CurrentVersion');
    {$ENDIF}
    ServerDirectory := R.ReadString('ServerDirectory');
  finally
    R.Free;
  end;

  try

    if Guardian then
      Exe := 'ibguard'
    else
      Exe := 'ibserver';

    ProcessHandle := CmdLineExecAsync(ServerDirectory + Exe);
    try
      if Guardian then
        case WaitForInputIdle(ProcessHandle,IBStartTimeout) of
          WAIT_TIMEOUT: ShowMessage('Interbase start timeout interval (' + IntToStr(IBStartTimeout) + ' msec) ellapsed.');
          $FFFFFFFF: ShowMessage('WaitForInputIdle() error.')
        end;
    finally
      CloseHandle(ProcessHandle)
    end;

    if Guardian then
      Sleep(AfterGuardianTimeout);

  except
    on E: Exception do
      ShowMessage(E.Message)
  end

end;

procedure TInterbaseAutostart.Start;
begin
  IBStart(false)
end;

{ TInterbaseGuardianAutostart }

procedure TInterbaseGuardianAutostart.Start;
begin
  IBStart(true)
end;

end.
