unit OnlyInstance;

interface

uses
  Windows, SysUtils, Classes, Forms;

type

  TOnlyInstance = class(TComponent)
  private
    FMutex: THandle;
    FMutexName: string;
    FOnlyInstance: boolean;
    procedure SetMutexName(const Value: string);

    procedure FreeMutex;
    function GetIsOnly: boolean;
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property IsOnly: boolean read GetIsOnly;
  published
    property MutexName: string read FMutexName write SetMutexName;
  end;

  // TODO: OnLoad event.

procedure Register;

implementation

{$R *.DCR}

procedure Register;
begin
  RegisterComponents('BUtil', [TOnlyInstance]);
end;

{ TOnlyInstance }

procedure TOnlyInstance.FreeMutex;
begin
  if FMutex <> 0 then
  begin
    CloseHandle(FMutex);
    FMutex := 0;
  end
end;

constructor TOnlyInstance.Create(AOwner: TComponent);
begin
  inherited;
  FMutex := 0;
  if (csDesigning in ComponentState) and Assigned(Owner) and (Owner is TForm) then
    FMutexName := TForm(Owner).Caption
end;

destructor TOnlyInstance.Destroy;
begin
  FreeMutex;
  inherited
end;

procedure TOnlyInstance.SetMutexName(const Value: string);
begin
  FMutexName := Value;

  FreeMutex;

  if not (csDesigning in ComponentState) and (Value<>'') then
  begin
    FMutex := OpenMutex(0,LongBool(false),PChar(Value));
    if FMutex = 0 then
    begin
      FOnlyInstance := true;
      FMutex := CreateMutex(nil,LongBool(false),PChar(Value));
      if FMutex = 0 then
        raise Exception.Create('TOnlyInstance.SetMutexName() CreateMutex() failed.')
    end
    else
      FOnlyInstance := false
  end
end;

function TOnlyInstance.GetIsOnly: boolean;
begin
  MutexName := FMutexName;  // To refresh.
  if (FMutex = 0) then
    raise Exception.Create('TOnlyInstance.GetOnlyInstance() Set the Mutex name!');
  result := FOnlyInstance
end;

end.
